function OUT = FNVcalc(Cm,Cd,D,h,S,W,Tvec,Zvec,seed)
% OUT = FNVcalc(Cm,Cd,D,h,S,W,Tvec,Zvec,seed)
% Input:  S = specter values, W = wave frequencies
% Output: OUT is a structure with the given fields: LM, LF, Fdist, FM
% Calculates the FNV load model
% FM is consistent with FNV equation
% Fdist is FNV "lumped" to nodes
% LF = lumped force, sum of Fdist for each time step
% LM = lumped moment, moment by Fdist for each time step
% LF and LM can be compared to FM, which validate the "lumping" process
% Call: K = wave_number(W,h)




cd ..
% Constants
rho = 1025;
g   = 9.81;
%% Create values from specter
cd diverse;
K = wave_number(W,h);  % Wave numbers
rng(seed);                      % set seed
dW  = W(2)-W(1);                % evenly spaced spectral components
g   = 9.81;
Za  = sqrt(2*S*dW);             % amplitudes from spectrum
eps = 2*pi*rand(1,length(W));   % random phase angles, the seed is set under input

%% Structure
Znod = Zvec;
temp = max(find(Zvec<0));
Zvec = [Zvec(1:temp) 0];
if isempty(temp); 'Zvec must contain position zero!'; return; end

% Create irregular functions
clear u utz wt w u uz a
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Distributed kinematics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lam  = 2*pi./K;                    % lam goes from long to short
tinf = min(find(lam  <  h/20));    % tinf = index of the largest lam shorter than h/20
if isempty(tinf)
    tinf = length(K)+1;
    lam  = [];
end
U    = zeros(length(Zvec),length(Tvec)); A = U;
temp = K*0;
for i = 1:length(Zvec)
    z    = Zvec(i);
    temp1(1:tinf-1) = W(1:tinf-1).*Za(1:tinf-1).* ...
                      cosh(K(1:tinf-1)*(z+h))./sinh(K(1:tinf-1)*h); % finite water
	temp2(1:tinf-1) = W(1:tinf-1).*Za(1:tinf-1).* ...
                      sinh(K(1:tinf-1)*(z+h))./sinh(K(1:tinf-1)*h); % finite water
	if (1-isempty(lam))
	temp1(tinf:end) = W(tinf:end).*Za(tinf:end).*exp(K(tinf:end)*z); % deep water
    temp2(tinf:end) = W(tinf:end).*Za(tinf:end).*exp(K(tinf:end)*z); % deep water
    end
    for j = 1:length(Tvec)
        t      = Tvec(j);
        w(i,j)  = sum(   temp2.*   sin(-W*t + eps));
        a(i,j)  = sum(W.*temp1.*   sin(-W*t + eps));
        uz(i,j) = sum(   temp2.*K.*cos(-W*t + eps));
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Kinematics at z = 0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i = 1:length(Tvec)
    t = Tvec(i);
    % Wave kinematics
    u(i)  = sum(     temp1.*   cos(-W*t + eps)  );
    az(i) = sum(  W.*temp2.*K.*sin(-W*t + eps)  );
    wt(i) = sum( -W.*temp2.*   cos(-W*t + eps)  );
    eta(i)= sum( Za.*          cos(-W*t + eps)  );
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Forces in time domain
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
BBB = @(x) 4;  
for i = 1:length(Tvec)
    dF1   = Cm*rho*pi*D^2/4*a(:,i);
    F1(i) =       trapz(Zvec',dF1);                         % Linear: Mass term in Morison equation!
    
    F2(i) = Cm*pi*rho*D^2/4  *a(end,i)*eta(i);              % Taylor expansion of dF1 to first order
    dF2   =    pi*rho*D^2/4  *uz(:,i).*w(:,i);              % Second order load
    F2(i) =       trapz(Zvec',dF2) + F2(i);
    
    F3a(i)= Cm*pi*rho*D^2/8* az(i)*eta(i)^2;                % Taylor expansion of dF1 to second order
    F3b(i)=    pi*rho*D^2/4*eta(i)*  ( uz(end,i)*w(end,i)); % Taylor expansion of dF2 to first order 
    F3c(i)= Cm*pi*rho*D^2/4*a(end,i)*( -eta(i)/g*wt(i) );   % Taylor, linear of dF1 in z = [eta1 eta1+eta2]          --> point load
    F3d(i)=    pi*rho*D^2/4/g*u(i)^2*a(end,i)*BBB(2*h/D);   % Non-linear diffraction potential, works close to z=eta --> point load
end
FM(1,:) = F1+F2+F3a+F3b+F3c+F3d;
%% Moments in time domain
GGG = @(x) 4.95037;  
for i = 1:length(Tvec)
    dF1   = Cm*pi*rho*D^2/4 *a(:,i);
    M1(i) = trapz(Zvec,(Zvec+h).*dF1');                      % Linear: Mass term in Morison equation!
    
    M2(i) = Cm*pi*rho*D^2/4 *a(end,i)*eta(i)^2/2;
    M2(i) = Cm*pi*rho*D^2/4 *a(end,i)*eta(i)                *h    + M2(i);
    dF2   =    pi*rho*D^2/4 *uz(:,i).*w(:,i);
    M2(i) = trapz(Zvec,(Zvec+h).*dF2')                            + M2(i);
    
    M3a(i)= Cm*pi*rho*D^2/12*az(i)*eta(i)^3;
    M3a(i)= Cm*pi*rho*D^2/8 *az(i)*eta(i)^2                 *h    + M3a(i);
    M3b(i)=    pi*rho*D^2/8 *w(end,i)*uz(end,i)*eta(i)^2;
    M3b(i)=    pi*rho*D^2/4 *w(end,i)*uz(end,i)*eta(i)      *h    + M3b(i);
    M3c(i)= Cm*pi*rho*D^2/4*a(end,i)*( -eta(i)*wt(i)/g )*(eta(i)+h); % works at the free surface!
    M3d(i)=    pi*rho*D^2/4/g*u(i)^2*a(end,i)*BBB(2*h/D)*(eta(i)+h); % works at the free surface!
   %M3e(i)=   -pi*rho*D^3/8/g*u(i)^2*a(end,i)*GGG(2*h/D);  % not possible to include in lumped loads
end
FM(2,:) = M1+M2+M3a+M3b+M3c+M3d;%+M3e;
%% Load Lumping: Distributed in z = [-h 0]
out = zeros(length(Znod),length(Tvec));
for i = 1:length(Tvec)
    dF1 = Cm*rho*pi*D^2/4* a(:,i);
    dF2 =    rho*pi*D^2/4*uz(:,i).*w(:,i);
    Fdist = dF1 + dF2;
        z1 = Zvec(1);
        z2 = (Zvec(2)+Zvec(1))/2;
        out(1,i)  = (z2-z1)*Fdist(1);
    for j = 2:(length(Zvec)-1)
        z1 = z2;
        z2 = (Zvec(j+1)+Zvec(j))/2;   % mean of the two points
        out(j,i)  = (z2-z1)*Fdist(j);
    end
        z1 = z2;
        z2 = 0;
        out(j+1,i)= (z2-z1)*Fdist(end);
end
%% Load Lumping: Distributed in z = [0 eta(i)]
loc0 = min(find(Znod==0));
for i = 1:length(out(1,:))
    eta(i)
    i
    loc    = min(find(Znod>eta(i))); % lowest point above surface on monopile!
    distF2 =      Cm*pi*rho*D^2/4*a(end,i);
    dF2    =         pi*rho*D^2/4*w(end,i)*uz(end,i);
    Fdist  = @(z) Cm*pi*rho*D^2/4*az(i)*z  +distF2+dF2;  % distF3a
    Fsur   = F3c(i) + F3d(i);
if eta(i)>= 0
loc  = loc-1;        % highest point below surface on monopile!
Zvec = Znod(loc0:loc);
    N  = length(Zvec);
    k  = loc0;       % indices in out!
    j  = 1;          % indices in Zvec!
    z1 = Zvec(j);
    while N > 1  
        z2       = (Zvec(j+1)+Zvec(j))/2;    
        out(k,i) = (z2-z1)* Fdist(Zvec(j))                + out(k,i);
        k        = k+1;
        j        = j+1;
        N        = N-1;
        z1       = z2;
    end
    z2 = Zvec(j);
    out(k,i)   = (z2-z1)* Fdist(Zvec(j))                  + out(k,i);
    z1 = z2;
	z2 = eta(i);
    out(k,i)   = (z2-z1)*(Fdist(Zvec(j))+Fdist(eta(i)))/2 + out(k,i)+Fsur;
else
Zvec = Znod(loc:loc0);
    N = length(Zvec);
    k = loc;
    j = 1;
    z1 = eta(i);
    z2 = Zvec(1);
    out(k,i)   = (z1-z2)*(Fdist(Zvec(j))+Fdist(eta(i)))/2 + out(k,i)+Fsur;
    z1 = z2;
    while N > 1  
        z2       = (Zvec(j+1)+Zvec(j))/2;    
        out(k,i) = (z1-z2)* Fdist(Zvec(j))                + out(k,i);
        k  = k+1;
        j  = j+1;
        N  = N-1;
        z1 = z2;
    end
    z2      = Zvec(j);
    out(k,i)= (z1-z2)* Fdist(Zvec(j))                     + out(k,i);
end
end
%%
LF = sum(out);
LM = zeros(1,length(LF));
for i = 1:length(LF)
LM(i) = sum((Znod+h).*out(:,i)');
end

%% Create output
OUT.LM    = LM;
OUT.LF    = LF;
OUT.Fdist = out;
OUT.FM    = FM;